<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Console;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Output\OutputInterface;
use Neklo\Scheduler\Api\Data\JobInterface;

class RunJob extends Command
{
    /**
     * @var \Neklo\Scheduler\Model\ScheduleFactory
     */
    private $scheduleFactory;

    /**
     * @var \Neklo\Scheduler\Model\ResourceModel\Job
     */
    private $jobFactory;

    /**
     * @var \Magento\Framework\App\State
     */
    private $state;

    public function __construct(
        \Neklo\Scheduler\Model\ScheduleFactory $scheduleFactory,
        \Neklo\Scheduler\Model\JobFactory $jobFactory,
        \Magento\Framework\App\State $state
    ) {
        $this->scheduleFactory = $scheduleFactory;
        $this->jobFactory = $jobFactory;
        $this->state = $state;
        parent::__construct();
    }

    protected function configure()
    {
        $this->setName('scheduler:cron:run');
        $this->setDescription('Run cron job by job name');
        $this->addArgument('job_name', InputArgument::REQUIRED, __('Job Name'));

        parent::configure();
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        try {
            $this->state->setAreaCode(\Magento\Framework\App\Area::AREA_CRONTAB);
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $output->writeln("<info>Area code is already set.</info>");
        }
        $job = $this->jobFactory->create()->load($input->getArgument('job_name'), JobInterface::JOB_NAME);
        if ($job->getId() === null) {
            $output->writeln("Current job does not exist");
            return;
        }

        $schedule = $this->scheduleFactory->create()
            ->initializeFromJob($job)
            ->setJob($job)
            ->setScheduledReason("Run From Console");

        $schedule->runNow(false)->save();

        $messages = $schedule->getMessages();

        $key = $job->getName();
        if (in_array(
            $schedule->getStatus(),
            [
                \Neklo\Scheduler\Model\Schedule::STATUS_SUCCESS,
                \Neklo\Scheduler\Model\Schedule::STATUS_DIDNTDOANYTHING
            ]
        )
        ) {
            $result = "Ran {$key} (Duration: {$schedule->getDuration()} sec)";
            $output->writeln($result);
            if ($messages) {
                $message = $key . " messages: {$messages}";
                $output->writeln($message);
            }
        } else {
            $result = "Error while running {$key}";
            $output->writeln($result);
            if ($messages) {
                $message = $key . ' messages: '. $messages;
                $output->writeln($message);
            }
        }
    }
}
